#![forbid(unsafe_code)]
#![deny(missing_docs)]

/*!
# Jargon

Jargon is a simple command line handler for Rust. Using a similar architecture to
[pico-args](https://github.com/RazrFalcon/pico-args), but *working* to be cleaner
and have more options — while being usable, unlike [clap](https://github.com/clap-rs/clap).

Jargon is aimed to create the least amount of handler code in a program but giving options
to handle any type of argument, including ones that don't start with `-` & subcommands.

## Examples

See [examples](examples/)
*/

mod error;
mod jargons;
mod keys;

pub use error::*;
pub use jargons::*;
pub use keys::*;

#[cfg(test)]
mod tests;
